
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the MVCI_CAN_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// MVCI_CAN_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef MVCI_CAN_EXPORTS
#define MVCI_CAN_API __declspec(dllexport)
#else
#define MVCI_CAN_API __declspec(dllimport)
#endif



/***  For BCB ***/
#ifdef	__cplusplus  // for C++ compile use
extern "C" {
#endif	/*	__cplusplus	*/


/* Common Return Code */
#define No_Err						0	//No Error
#define DEV_ModName_Err				1	//The Module Name Error
#define DEV_ModNotExist_Err			2	//The Module doesn't exist in this Port
#define DEV_PortNotExist_Err		3	//The Port doesn't Exist
#define DEV_PortInUse_Err			4	//The Port is in Used
#define DEV_PortNotOpen_Err			5	//The Port doesn't Open
#define CAN_ConfigFail_Err			6	//CAN Config Command Fail
#define CAN_HARDWARE_Err			7	//CAN Hardware Init Fail
#define CAN_PortNo_Err				8	//The Device doesn't support this CAN Port
#define CAN_FIDLength_Err			9	//The CAN Filter-ID Number exceed Max Number
#define CAN_DevDisconnect_Err		10	//The Connection of device is broken
#define CAN_TimeOut_Err				11	//The Config Command Timeout
#define CAN_ConfigCmd_Err			12	//The Config Command doesn't support
#define CAN_ConfigBusy_Err			13	//The Config Command is busy
#define CAN_RxBufEmpty				14	//The CAN Receive Buffer is empty
#define CAN_TxBufFull				15	//The CAN Send Buffer is full
#define CAN_UserDefISRNo_Err		16  //The User Defined ISR No Error (0~7)


/* Extra Return Code for mVCI_CAN.dll */
#define VCI_CAN_DLL_NotFound		-1	//VCI_CAN.dll is not exist
#define VCI_CAN_DLL_LoadFail		-2  //VCI_CAN.dll load fail



/************************      VCI_CAN API Variable      *************************/ 
//Common Define
#define CAN1						1
#define CAN2						2
#define CONFIG_MODE					0
#define COMM_MODE					1
#define TPFLOATMS(fVal)				((DWORD)(fVal*10)+100000000)

//Device Type
#define I7565H1						1
#define I7565H2						2

//CANMsg Define
#define MODE_11BIT					0
#define MODE_29BIT					1
#define RTR_0						0
#define RTR_1						1
#define DLC_0						0
#define DLC_1						1
#define DLC_2						2
#define DLC_3						3
#define DLC_4						4
#define DLC_5						5
#define DLC_6						6
#define DLC_7						7
#define DLC_8						8
	
//UserDefISR Define
#define ISRNO_0						0
#define ISRNO_1						1
#define ISRNO_2						2
#define ISRNO_3						3
#define ISRNO_4						4
#define ISRNO_5						5
#define ISRNO_6						6
#define ISRNO_7						7
#define ISR_CANPORT_ALL				0
#define ISR_CANMODE_ALL				2
#define ISR_CANID_ALL				0

//CAN Baudrate
#define CANBaud_5K					5000
#define CANBaud_10K					10000
#define CANBaud_20K					20000
#define CANBaud_40K					40000
#define CANBaud_50K					50000
#define CANBaud_80K					80000
#define CANBaud_100K				100000
#define CANBaud_125K				125000
#define CANBaud_200K				200000
#define CANBaud_250K				250000
#define CANBaud_400K				400000
#define CANBaud_500K				500000
#define CANBaud_600K				600000
#define CANBaud_800K				800000
#define CANBaud_1000K				1000000


/* ICPDAS VCICAN Series Message Structure Format */
//[1]. VCICAN - Module Info Format
typedef  struct  _VCI_MODULE_INFO{
	char	Mod_ID[12];
	char	FW_Ver[12];
	char	HW_SN[16];
} _VCI_MODULE_INFO, *PVCI_MOD_INFO; 

//[2]. VCICAN - CAN Parameter (CANBaud: Unit(bps))
typedef  struct  _VCI_CAN_PARAM{
	BYTE	DevPort;
	BYTE	DevType;
	DWORD	CAN1_Baud;
	DWORD	CAN2_Baud;
} _VCI_CAN_PARAM, *PVCI_CAN_PARAM;

typedef  struct  _VCI_CAN_PARAM_EX{
	BYTE	DevPort;
	BYTE	DevType;
	DWORD	CAN1_Baud;
	DWORD	CAN2_Baud;
	BYTE	CAN1_T2Val;
	BYTE	CAN2_T2Val;
	BYTE	Reserved[32];
} _VCI_CAN_PARAM_EX, *PVCI_CAN_PARAM_EX;

//[3]. VCICAN - CAN Message Format
typedef  struct  _VCI_CAN_MSG{
	BYTE	Mode;		
	BYTE	RTR;
	BYTE	DLC;
	BYTE	Reserved;
	DWORD	ID;
	DWORD	TimeL;
	DWORD	TimeH;
	BYTE	Data[8];
} _VCI_CAN_MSG, *PVCI_CAN_MSG;

//[4]. VCICAN - CAN FilterID Format
typedef  struct  _VCI_CAN_FilterID{
	WORD	SSFF_Num;		
	WORD	GSFF_Num;
	WORD	SEFF_Num;
	WORD	GEFF_Num;
	WORD	SSFF_FID[512];
	DWORD	GSFF_FID[512];
	DWORD	SEFF_FID[512];
	DWORD	GEFF_FID[512];
} _VCI_CAN_FilterID, *PVCI_CAN_FID;

//[5]. VCICAN - CAN Status Format
typedef struct _VCI_CAN_STATUS{
	DWORD	CurCANBaud;
	BYTE	CANReg;
	BYTE	CANTxErrCnt;
	BYTE	CANRxErrCnt;
	BYTE	MODState;
	DWORD	Reserved;
} _VCI_CAN_STATUS, *PVCI_CAN_STATUS;


//[1] Init Function
typedef int (__stdcall *PFN_VCI_OpenCAN)(PVCI_CAN_PARAM);
typedef int (__stdcall *PFN_VCI_OpenCAN_NoStruct)(BYTE, BYTE, DWORD, DWORD);
typedef int (__stdcall *PFN_VCI_CloseCAN)(BYTE);
//[2] Module Config Function
typedef int (__stdcall *PFN_VCI_Set_CANFID)(BYTE, PVCI_CAN_FID);
typedef int (__stdcall *PFN_VCI_Set_CANFID_NoStruct)(BYTE, WORD, WORD, WORD, WORD, WORD SSFF_FID[512], DWORD GSFF_FID[512], DWORD SEFF_FID[512], DWORD GEFF_FID[512]);
typedef int (__stdcall *PFN_VCI_Set_CANFID_AllPass)(BYTE);
typedef int (__stdcall *PFN_VCI_Get_CANFID)(BYTE, PVCI_CAN_FID);
typedef int (__stdcall *PFN_VCI_Get_CANFID_NoStruct)(BYTE, WORD*, WORD*, WORD*, WORD*, WORD SSFF_FID[512], DWORD GSFF_FID[512], DWORD SEFF_FID[512], DWORD GEFF_FID[512]);
typedef int (__stdcall *PFN_VCI_Get_CANStatus)(BYTE, PVCI_CAN_STATUS);
typedef int (__stdcall *PFN_VCI_Get_CANStatus_NoStruct)(BYTE, DWORD*, BYTE*, BYTE*, BYTE*, BYTE*);
typedef int (__stdcall *PFN_VCI_Clr_BufOverflowLED)(BYTE);
typedef int (__stdcall *PFN_VCI_Get_MODInfo)(PVCI_MOD_INFO);
typedef int (__stdcall *PFN_VCI_Get_MODInfo_NoStruct)(char Mod_ID[12], char FW_Ver[12], char HW_SN[16]);
typedef int (__stdcall *PFN_VCI_Rst_MOD)(void);
typedef int (__stdcall *PFN_VCI_Set_MOD_Ex)(BYTE CfgData[512]);
//[3] Communication Function
typedef int (__stdcall *PFN_VCI_SendCANMsg)(BYTE, PVCI_CAN_MSG);
typedef int (__stdcall *PFN_VCI_SendCANMsg_NoStruct)(BYTE, BYTE, BYTE, BYTE, DWORD, BYTE Data[8]);
typedef int (__stdcall *PFN_VCI_RecvCANMsg)(BYTE, PVCI_CAN_MSG);
typedef int (__stdcall *PFN_VCI_RecvCANMsg_NoStruct)(BYTE, BYTE*, BYTE*, BYTE*, DWORD*, DWORD*, DWORD*, BYTE Data[8]);
typedef int (__stdcall *PFN_VCI_EnableHWCyclicTxMsg)(BYTE, PVCI_CAN_MSG, DWORD, DWORD);
typedef int (__stdcall *PFN_VCI_EnableHWCyclicTxMsg_NoStruct)(BYTE, BYTE, BYTE, BYTE, DWORD, BYTE Data[8], DWORD, DWORD);
typedef int (__stdcall *PFN_VCI_DisableHWCyclicTxMsg)(void);
typedef int (__stdcall *PFN_VCI_EnableHWCyclicTxMsgNo)(BYTE, BYTE, BYTE, BYTE, DWORD, BYTE Data[8], DWORD, DWORD, BYTE);
typedef int (__stdcall *PFN_VCI_EnableHWCyclicTxMsgNo_Ex)(BYTE, BYTE, BYTE, BYTE, DWORD, BYTE Data[8], DWORD, DWORD, BYTE, BYTE, DWORD, DWORD);
typedef int (__stdcall *PFN_VCI_DisableHWCyclicTxMsgNo)(BYTE);
//[4] Software Buffer Function
typedef int (__stdcall *PFN_VCI_Get_RxMsgCnt)(BYTE, DWORD*);
typedef int (__stdcall *PFN_VCI_Get_RxMsgBufIsFull)(BYTE, BYTE*);
typedef int (__stdcall *PFN_VCI_Clr_RxMsgBuf)(BYTE);
typedef int (__stdcall *PFN_VCI_Get_TxMsgCnt)(BYTE, DWORD*);
typedef int (__stdcall *PFN_VCI_Clr_TxMsgBuf)(BYTE);
typedef int (__stdcall *PFN_VCI_Get_TxSentCnt)(BYTE, DWORD*);
typedef int (__stdcall *PFN_VCI_Clr_TxSentCnt)(BYTE);
//[5] UserDefine ISR Function
typedef	int (__stdcall *PFN_VCI_Set_UserDefISR)(BYTE, BYTE, BYTE, DWORD, void (*)());
typedef	int (__stdcall *PFN_VCI_Clr_UserDefISR)(BYTE);
typedef	int (__stdcall *PFN_VCI_Get_ISRCANData)(BYTE, BYTE*, BYTE Data[8]);
typedef	int (__stdcall *PFN_VCI_Get_ISRCANData_Ex)(BYTE, DWORD*, BYTE*, BYTE Data[8]);
//[6] Other Function
typedef int (__stdcall *PFN_VCI_Get_DllVer)(void);
typedef int (__stdcall *PFN_VCI_DoEvents)(void);
//[7] Extended Function
typedef int (__stdcall *PFN_VCI_OpenCAN_Ex)(PVCI_CAN_PARAM_EX);
typedef int (__stdcall *PFN_VCI_OpenCAN_NoStruct_Ex)(BYTE, BYTE, DWORD, DWORD, BYTE, BYTE);
typedef int (__stdcall *PFN_VCI_Get_CANBaud_BitTime)(BYTE, BYTE*, BYTE*, BYTE*);


// Variable is exported from mVCI_CAN.dll
extern MVCI_CAN_API char* ErrMessage[32];
extern MVCI_CAN_API char* mErrMessage[5];
extern MVCI_CAN_API DWORD CANBaud_Arr[15];

// Class is exported from mVCI_CAN.dll
class MVCI_CAN_API CMVCI_CAN {
public:
	CMVCI_CAN(void);
	~CMVCI_CAN(void);
	
	//[1] Init Function
	PFN_VCI_OpenCAN mVCI_OpenCAN;
	PFN_VCI_OpenCAN_NoStruct mVCI_OpenCAN_NoStruct;
	PFN_VCI_CloseCAN mVCI_CloseCAN;
	//[2] Module Config Function
	PFN_VCI_Set_CANFID mVCI_Set_CANFID;
	PFN_VCI_Set_CANFID_NoStruct mVCI_Set_CANFID_NoStruct;
	PFN_VCI_Set_CANFID_AllPass mVCI_Set_CANFID_AllPass;
	PFN_VCI_Get_CANFID mVCI_Get_CANFID;
	PFN_VCI_Get_CANFID_NoStruct mVCI_Get_CANFID_NoStruct;
	PFN_VCI_Get_CANStatus mVCI_Get_CANStatus;
	PFN_VCI_Get_CANStatus_NoStruct mVCI_Get_CANStatus_NoStruct;
	PFN_VCI_Clr_BufOverflowLED mVCI_Clr_BufOverflowLED;
	PFN_VCI_Get_MODInfo mVCI_Get_MODInfo;
	PFN_VCI_Get_MODInfo_NoStruct mVCI_Get_MODInfo_NoStruct;
	PFN_VCI_Rst_MOD mVCI_Rst_MOD;
	PFN_VCI_Set_MOD_Ex mVCI_Set_MOD_Ex;
	//[3] Communication Function
	PFN_VCI_SendCANMsg mVCI_SendCANMsg;
	PFN_VCI_SendCANMsg_NoStruct mVCI_SendCANMsg_NoStruct;
	PFN_VCI_RecvCANMsg mVCI_RecvCANMsg;
	PFN_VCI_RecvCANMsg_NoStruct mVCI_RecvCANMsg_NoStruct;
	PFN_VCI_EnableHWCyclicTxMsg mVCI_EnableHWCyclicTxMsg;
	PFN_VCI_EnableHWCyclicTxMsg_NoStruct mVCI_EnableHWCyclicTxMsg_NoStruct;
	PFN_VCI_DisableHWCyclicTxMsg mVCI_DisableHWCyclicTxMsg;
	PFN_VCI_EnableHWCyclicTxMsgNo mVCI_EnableHWCyclicTxMsgNo;
	PFN_VCI_EnableHWCyclicTxMsgNo_Ex mVCI_EnableHWCyclicTxMsgNo_Ex;
	PFN_VCI_DisableHWCyclicTxMsgNo mVCI_DisableHWCyclicTxMsgNo;
	//[4] Software Buffer Function
	PFN_VCI_Get_RxMsgCnt mVCI_Get_RxMsgCnt;
	PFN_VCI_Get_RxMsgBufIsFull mVCI_Get_RxMsgBufIsFull;
	PFN_VCI_Clr_RxMsgBuf mVCI_Clr_RxMsgBuf;
	PFN_VCI_Get_TxMsgCnt mVCI_Get_TxMsgCnt;
	PFN_VCI_Clr_TxMsgBuf mVCI_Clr_TxMsgBuf;
	PFN_VCI_Get_TxSentCnt mVCI_Get_TxSentCnt;
	PFN_VCI_Clr_TxSentCnt mVCI_Clr_TxSentCnt;
	//[5] UserDefine ISR Function
	PFN_VCI_Set_UserDefISR mVCI_Set_UserDefISR;
	PFN_VCI_Clr_UserDefISR mVCI_Clr_UserDefISR;
	PFN_VCI_Get_ISRCANData mVCI_Get_ISRCANData;
	PFN_VCI_Get_ISRCANData_Ex mVCI_Get_ISRCANData_Ex;
	//[6] Other Function
	PFN_VCI_Get_DllVer mVCI_Get_DllVer;
	PFN_VCI_DoEvents mVCI_DoEvents;
	//[7] Extended Function
	PFN_VCI_OpenCAN_Ex mVCI_OpenCAN_Ex;
	PFN_VCI_OpenCAN_NoStruct_Ex mVCI_OpenCAN_NoStruct_Ex;
	PFN_VCI_Get_CANBaud_BitTime mVCI_Get_CANBaud_BitTime;

	int InitDLL(void);	
	
private:
	int FreeDLL(void);
	int LoadDLL(char* DLLPathName);
	HMODULE hMod;
	DWORD DLLIdx;
};


/***  For BCB ***/
#ifdef	__cplusplus
}
#endif	/*	__cplusplus	*/

